/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include"mainwindow.h" // class's header file
#include<ctime>
#include<iostream>
#include<fstream>
#include<vector>
#include<stdio.h>
#include<libintl.h>
#include"globals.h"
#include"defaults.h"
#include"prefdialog.h"
#include"statdialog.h"
#include"viewmodel.h"

// class constructor
PCalMainWindow::PCalMainWindow(BaseObjectType* cobject, 
                               const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
: Gtk::Window(cobject),
  m_refGlade(refGlade),
  cal_current_date(Glib::Date(1,Glib::Date::JANUARY,1900)),
  pAboutDialog(0),
  pPrefDialog(0),
  pStatDialog(0),
  pIEDialog(0),
  pPopupMenu(0),
  addPopupMenuItem(0),
  deletePopupMenuItem(0),
  addMenstruationMenuItem(0),
  addPregnancyMenuItem(0),
  addBirthMenuItem(0),
  deleteMenstruationMenuItem(0),
  openFileChooser(0),
  openFileChooserButtonBox(0),
  openFileChooserOKButton(0),
  saveFileChooser(0),
  saveFileChooserButtonBox(0),
  saveFileChooserOKButton(0),
  today_year_day(0),
  notesColumns(0),
  notesModel(0),
  notesTreeView(0),
  notesTextView(0),
  noteAppendButton(0),
  noteRemoveButton(0),
  noteAppendMenuItem(0),
  noteRemoveMenuItem(0),
  keyboardSelected(false),
  activatingNote(false),
  resettingNoteText(false),
  editMenuitem(0)
{
    current_file = "";
    
    // get the options
    calOptions = new PCalOptions(this);

    // start the engine
    engine = new PCalEngine(calOptions->bufferDays,
                   calOptions->badFeelDaysBefore,
                   calOptions->badFeelDaysAfter,
                   calOptions->regularMinLength,
                   calOptions->regularMaxLength);
    
    // initialize the widgets references
    refGlade->get_widget("timerLabel", timerLabel);
    refGlade->get_widget("toolbar1", mainToolbars[0]);
    refGlade->get_widget("toolbar2", mainToolbars[1]);
    // set the toolbar style
    refreshToolbar();
    refGlade->get_widget("calendarTreeView", calendarTreeView);
    refGlade->get_widget("NotesTreeView", notesTreeView);
    refGlade->get_widget("NotesTextView", notesTextView);
    refGlade->get_widget("monthLabel", monthLabel);
    refGlade->get_widget("yearLabel", yearLabel);
    refGlade->get_widget("DayInfoLabel", dayInfoLabel);
    refGlade->get_widget("PeriodDayNumberLabel", periodDayNumberLabel);
    refGlade->get_widget("FertilityLabel", fertilityLabel);
    refGlade->get_widget("BadFeelingsLabel", badFeelingsLabel);
    refGlade->get_widget("PredictionLabel", predictionLabel);
    refGlade->get_widget("statusLabel", statusLabel);    
    refGlade->get_widget("AddMenstruationItem", addMenstruationMenuItem);
    refGlade->get_widget("RecordPregnancyItem", addPregnancyMenuItem);
    refGlade->get_widget("RecordBirthItem", addBirthMenuItem);
    refGlade->get_widget("DeleteMenstruationItem", deleteMenstruationMenuItem);
    refGlade->get_widget("InfoNotebook", infoNotebook);
    refGlade->get_widget("PregnancyWeekLabel", pregnancyWeekLabel);
    refGlade->get_widget("EstimatedBirthLabel", estimatedBirthLabel);
    refGlade->get_widget("editMenuitem", editMenuitem);
    editMenuitem->hide();
    Gtk::ToolButton* pButton = 0;
    refGlade->get_widget("prevMonthButton", pButton);
    if(pButton) {
        pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::prevMonth));
    }
    Gtk::MenuItem* pMenuItem = 0;
    refGlade->get_widget("prevMonthMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::prevMonth));
    }

    pButton = 0;
    refGlade->get_widget("nextMonthButton", pButton);
    if(pButton) {
        pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::nextMonth));
    }
    pMenuItem = 0;
    refGlade->get_widget("nextMonthMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::nextMonth));
	}

    pButton = 0;
    refGlade->get_widget("prevYearButton", pButton);
    if(pButton) {
        pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::prevYear));
    }
    pMenuItem = 0;
    refGlade->get_widget("prevYearMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::prevYear));
    }

    pButton = 0;
    refGlade->get_widget("nextYearButton", pButton);
    if(pButton) {
        pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::nextYear));
    }
    pMenuItem = 0;
    refGlade->get_widget("nextYearMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::nextYear));
    }

    pButton = 0;
    refGlade->get_widget("todayButton", pButton);   
    if(pButton) {
        pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::today));
    }
    pMenuItem = 0;
    refGlade->get_widget("todayMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::today));
    }
    pMenuItem = 0;
    refGlade->get_widget("SelectDateMenuItem", pMenuItem);
    if(pMenuItem) {
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_jump_to_date_item));
    }
    
    refGlade->get_widget("NoteAppendMenuItem", noteAppendMenuItem);
    if(noteAppendMenuItem) {
      noteAppendMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_note_append_item));
    }
    refGlade->get_widget("NoteAppendButton", noteAppendButton);
    if(noteAppendButton) {
        noteAppendButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::on_note_append_item));
    }
    refGlade->get_widget("NoteRemoveMenuItem", noteRemoveMenuItem);
    if(noteRemoveMenuItem) {
      noteRemoveMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_note_remove_item));
    }
    refGlade->get_widget("NoteRemoveButton", noteRemoveButton);
    if(noteRemoveButton) {
        noteRemoveButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::on_note_remove_item));
    }

    refreshLegend();
    
    calModel = new MonthCalendarModel(new MonthCalendarColumns);
    
    calendarTreeView->set_model(
       Glib::RefPtr<MonthCalendarModel>(calModel));
    calendarTreeView->get_selection()->set_select_function( sigc::mem_fun(*this, &PCalMainWindow::select_date_function) );
    calModel->initialize(this);

    calendarTreeView->signal_button_press_event().connect_notify( sigc::mem_fun(*this, &PCalMainWindow::on_calendar_click));
    //this->signal_key_press_event().connect_notify( sigc::mem_fun(*this, &PCalMainWindow::on_calendar_key_press));
    
    model_current_date = &(calModel->current_date);
    model_today_date =  &(calModel->todayDate);
    
    // notes engine start
    notesColumns = new NotesModelColumns;
    notesModel = Gtk::ListStore::create(*notesColumns);
    notesModel->set_sort_func(notesColumns->m_col_date, sigc::mem_fun(*this, &PCalMainWindow::treeModelDatesCompare));
    notesModel->set_sort_column(notesColumns->m_col_date, Gtk::SORT_ASCENDING);
    notesTreeView->set_model(notesModel);
    notesTreeView->append_column("Date:", notesColumns->m_col_label);
    notesTreeView->get_selection()->set_select_function( sigc::mem_fun(*this, &PCalMainWindow::select_note_function));
    notesTextView->get_buffer()->signal_changed().connect( sigc::mem_fun(*this, &PCalMainWindow::on_note_text_changed));
    // connect the timer handler     
    Glib::signal_timeout().connect(sigc::mem_fun(*this, &PCalMainWindow::on_timer_tick), 1000);
    on_timer_tick();

    pMenuItem = 0;
    refGlade->get_widget("help_contentsMenuItem", pMenuItem);
    if(pMenuItem) {
      // !!! hide this menu - don't bother the user with help as for now
      pMenuItem->hide();
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::show_help_contents));
	}
    pMenuItem = 0;
    refGlade->get_widget("AboutMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_about_item));

    pMenuItem = 0;
    refGlade->get_widget("newMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_new_item));

    pButton = 0;
    refGlade->get_widget("newButton", pButton);
    if(pButton)
      pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::on_new_item));

    pMenuItem = 0;
    refGlade->get_widget("openMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_open_item));
    
    pButton = 0;
    refGlade->get_widget("openButton", pButton);
    if(pButton)
      pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::on_open_item));
    
    pMenuItem = 0;
    refGlade->get_widget("saveMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_save_item));

    pButton = 0;
    refGlade->get_widget("saveButton", pButton);
    if(pButton)
      pButton->signal_clicked().connect(sigc::mem_fun(*this, &PCalMainWindow::on_save_item));
    
    pMenuItem = 0;
    refGlade->get_widget("save_asMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_save_as_item));
    
    pMenuItem = 0;
    refGlade->get_widget("PrefMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_pref_item));

    pMenuItem = 0;
    refGlade->get_widget("StatMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_stat_item));
    
    pMenuItem = 0;
    refGlade->get_widget("ImportExportMenuItem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_ie_item));

    if(addMenstruationMenuItem) {
        addMenstruationMenuItem->set_sensitive(false);
        addMenstruationMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_add_menstruation_item));
    }
    if(addPregnancyMenuItem) {
        addPregnancyMenuItem->set_sensitive(false);
        addPregnancyMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_add_pregnancy_item));
    }
    if(addBirthMenuItem) {
        addBirthMenuItem->set_sensitive(false);
        addBirthMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_add_birth_item));
    }
    if(deleteMenstruationMenuItem) {
        deleteMenstruationMenuItem->set_sensitive(false);
        deleteMenstruationMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_delete_menstruation_item));
    }
    pMenuItem = 0;
    refGlade->get_widget("exitMenuitem", pMenuItem);
    if(pMenuItem)
      pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_quit_item));

    updateWindowTitle();
    
    // activate today date on starup
    calModel->current_date = calModel->todayDate;
    activateDate(calModel->current_date);
    // tree view should never have the focus it is useless for us
    calendarTreeView->unset_flags(Gtk::CAN_FOCUS);
    
    // GET THE WEEK DAY NAMES FROM LOCALE
    // I know this is SUNDAY
    Glib::Date tempDate(27,Glib::Date::JUNE,2004);
    
    for(int i=0;i<8;i++) {
        WDdays[i] = tempDate.format_string("%a");
        tempDate.add_days(1);
    }
}
    

// class destructor
PCalMainWindow::~PCalMainWindow()
{
    delete calOptions;
    delete engine;
}

void PCalMainWindow::refreshLegend()
{
    // fill the legend colors
    Gtk::Label* tempLabel = 0;
    Gtk::EventBox* tempEB = 0;
    
    m_refGlade->get_widget("LegendDay1Label", tempLabel);    
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->day1FGColor);
    }
 
    m_refGlade->get_widget("LegendDay1EventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->day1BGColor);
     
    }
    
    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendFertileLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->fertFGColor);
    }
    m_refGlade->get_widget("LegendFertileEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->fertBGColor);
    }
    
    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendPDay1Label", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->pday1FGColor);
    }
    m_refGlade->get_widget("LegendPDay1EventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->pday1BGColor);
    }
    
    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendPFertileLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->pfertFGColor);
    }
    m_refGlade->get_widget("LegendPFertileEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->pfertBGColor);
    }
    
    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendTodayLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->todayFGColor);
    }
    m_refGlade->get_widget("LegendTodayEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->todayBGColor);
    }

    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendPregnantLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->pregFGColor);
    }
    m_refGlade->get_widget("LegendPregnantEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->pregBGColor);
    }

    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendBirthLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->birthFGColor);
    }
    m_refGlade->get_widget("LegendBirthEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->birthBGColor);
    }

    tempLabel = 0;
    tempEB = 0;
    m_refGlade->get_widget("LegendPBirthLabel", tempLabel);
    if(tempLabel) {
        tempLabel->modify_fg(Gtk::STATE_NORMAL,calOptions->pbirthFGColor);
    }
    m_refGlade->get_widget("LegendPBirthEventBox", tempEB);
    if(tempEB) {
        tempEB->modify_bg(Gtk::STATE_NORMAL,calOptions->pbirthBGColor);
    }   
}

void PCalMainWindow::on_about_item() 
{    
  //Get the About Dialog
  if(pAboutDialog == 0) {
      Glib::RefPtr<Gnome::Glade::Xml> refXml;
      try
      {
        refXml = Gnome::Glade::Xml::create(glade_interface_path, "AboutWindow");
      }
      catch(const Gnome::Glade::XmlError& ex)
      {
        std::cerr << ex.what() << std::endl;
        return;
      } 
      refXml->get_widget("AboutWindow", pAboutDialog);
      if(pAboutDialog)
      {
        // Set the icon
        Gtk::Image *pImage = 0;
        refXml->get_widget("LogoImage", pImage);
        if(pImage) {
            pImage->set(app_logo_path);
        }
        
        //Get the Glade-instantiated Button, and connect a signal handler:
        Gtk::Button* pButton = 0;
        refXml->get_widget("OKButton", pButton);
        if(pButton)
        {
          pButton->signal_clicked().connect(sigc::mem_fun(*pAboutDialog, &Gtk::Dialog::hide));
        }      
        Gtk::Label* pLabel = 0;
        refXml->get_widget("progNameLabel", pLabel);
        if(pLabel)
        {
            pLabel->set_label(Glib::ustring("<big><b>") +
                             APP_NAME + 
                             "</b></big>");
        }
        pAboutDialog->set_transient_for(*this);
        pAboutDialog->run();
      }
  } else {
      pAboutDialog->set_transient_for(*this);
      pAboutDialog->run();
  }  
}

void PCalMainWindow::on_pref_item() 
{
  //Get the Preferences Dialog
  if(pPrefDialog == 0) {
      Glib::RefPtr<Gnome::Glade::Xml> refXml;
      try {
          refXml = 
          Gnome::Glade::Xml::create(glade_interface_path,
                 "PreferencesWindow");
      }
      catch(const Gnome::Glade::XmlError& ex)
      {
        std::cerr << ex.what() << std::endl;
        return;
      } 
      refXml->get_widget_derived("PreferencesWindow", pPrefDialog);
      if(pPrefDialog) {
          pPrefDialog->present(this);
      }
  } else {
      pPrefDialog->present(this);
  }  
}

void PCalMainWindow::on_ie_item() 
{
  //Get the Import/Export Dialog
  if(pIEDialog == 0) {
      Glib::RefPtr<Gnome::Glade::Xml> refXml;
      try {
          refXml = 
          Gnome::Glade::Xml::create(glade_interface_path,
                 "ImportExportWindow");
      }
      catch(const Gnome::Glade::XmlError& ex)
      {
        std::cerr << ex.what() << std::endl;
        return;
      } 
      refXml->get_widget_derived("ImportExportWindow", pIEDialog);
      if(pIEDialog) {
          pIEDialog->present(this);
      }
  } else {
      pIEDialog->present(this);
  }  
}

void PCalMainWindow::on_stat_item() 
{
  //Get the Statistics Dialog
  if(pStatDialog == 0) {
      Glib::RefPtr<Gnome::Glade::Xml> refXml;
      try {
          refXml = 
          Gnome::Glade::Xml::create(glade_interface_path,
                 "StatisticsWindow");
      }
      catch(const Gnome::Glade::XmlError& ex)
      {
        std::cerr << ex.what() << std::endl;
        return;
      } 
      refXml->get_widget_derived("StatisticsWindow", pStatDialog);
      if(pStatDialog) {
          pStatDialog->present(this);
      }
  } else {
      pStatDialog->present(this);
  }  
}

bool PCalMainWindow::on_timer_tick() 
{
    char s[51];
    std::time_t t;
    Glib::ustring ustr;
    std::time(&t);
    std::tm * tm = std::localtime(&t);
    std::strftime(s, 50, "%c", tm);
    ustr = Glib::locale_to_utf8(s);
    timerLabel->set_text(ustr);
    
    // need to switch today date for the calendar?
    if(today_year_day!=tm->tm_yday) {
        model_today_date->set_dmy(tm->tm_mday,
              (Glib::Date::Month)(tm->tm_mon+1),
              tm->tm_year+1900);
        today_year_day = tm->tm_yday;
    }
    return true;
}

bool PCalMainWindow::select_date_function(const Glib::RefPtr<Gtk::TreeModel>& model,
const Gtk::TreeModel::Path& path, bool) 
{
    if(!keyboardSelected) {
        int x, y, cell_x, cell_y;
        Gtk::TreeModel::Path path_s;
        Gtk::TreeViewColumn* column;
        
        calendarTreeView->get_pointer(x,y);
        calendarTreeView->get_path_at_pos(x, y, path_s, column, 
                                          cell_x, cell_y);
        Gtk::TreeRow row = *(model->get_iter(path));
    
        int column_pos = 0;
        for(;column_pos<7;column_pos++) {
            if(calendarTreeView->get_column(column_pos)==column) break;
        }
    
        MonthCalendarColumns * m_columns = Glib::RefPtr<MonthCalendarModel>::cast_dynamic(calendarTreeView->get_model())->columns;
        Glib::Date selDate = row[m_columns->date[column_pos]];
        *model_current_date = selDate;
        activateDate(selDate);
    }
    return false;
}

void PCalMainWindow::on_new_item()
{
    if(checkModified()) {
        engine->clear();
        current_file = "";
        updateWindowTitle();
        reload_notes_tree();
        refreshCalendarView();
    }
}
void PCalMainWindow::on_open_item()
{
    if(checkModified()) {        
        if(!openFileChooser) {
            Glib::RefPtr<Gnome::Glade::Xml> refXml;
            try {
                refXml = 
                Gnome::Glade::Xml::create(glade_interface_path,
                       "OpenFileChooser");
            }
            catch(const Gnome::Glade::XmlError& ex)
            {
              std::cerr << ex.what() << std::endl;
              return;
            } 
            refXml->get_widget("OpenFileChooser", openFileChooser);
            refXml->get_widget("dialog-action_area4", openFileChooserButtonBox);
            refXml->get_widget("OKButton", openFileChooserOKButton);
            openFileChooser->set_transient_for(*this);

            // add the filters
            Gtk::FileFilter filter_pcal;
            filter_pcal.set_name(PCAL_FILTER_NAME);
            filter_pcal.add_pattern(PCAL_FILTER_EXT);
            openFileChooser->add_filter(filter_pcal);
            Gtk::FileFilter filter_any;
            filter_any.set_name("Any files");
            filter_any.add_pattern("*");
            openFileChooser->add_filter(filter_any);
        }

        // rearrange the buttons according to the options
        if(Gtk::Dialog::alternative_button_order(get_screen())) {
            openFileChooserButtonBox->reorder_child(*openFileChooserOKButton, 0);
        } else {
            openFileChooserButtonBox->reorder_child(*openFileChooserOKButton, 1);
        }
        int result = openFileChooser->run();
        openFileChooser->hide();

        //Handle the response:
        switch(result)
        {
          case(Gtk::RESPONSE_OK):
          {
              // avoid the false modified state flag setting
              resettingNoteText = true;
              Glib::ustring c_file = Glib::filename_to_utf8(openFileChooser->get_filename());
              if(engine->loadFromFile(c_file)) {
                  current_file = c_file;
                  updateWindowTitle();
                  reload_notes_tree();
                  refreshCalendarView();
              } else {
                  Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
                              gettext("Can not load file:\n")+
                              c_file,
				              false,
                              Gtk::MESSAGE_ERROR,
                              Gtk::BUTTONS_OK,
                              true);
                  edialog->set_transient_for(*this);
                  edialog->run();
                  delete edialog;
              }
              resettingNoteText = false;
              break;
            }
        }
    }
}

void PCalMainWindow::on_save_item()
{
    if(current_file=="") on_save_as_item();
    else {
        if(!engine->saveToFile(current_file)) {
            Gtk::MessageDialog *dialog = new class Gtk::MessageDialog(
                        gettext("Can not save to file: \n")+
                        current_file,
			            false,
                        Gtk::MESSAGE_ERROR,
                        Gtk::BUTTONS_OK,
                        true);    
            dialog->set_transient_for(*this);
            dialog->run();
            delete dialog;
        }
    }
}

void PCalMainWindow::on_save_as_item()
{
    if(!saveFileChooser) {
      Glib::RefPtr<Gnome::Glade::Xml> refXml;
      try {
          refXml = 
          Gnome::Glade::Xml::create(glade_interface_path,
                 "SaveFileChooser");
      }
      catch(const Gnome::Glade::XmlError& ex)
      {
        std::cerr << ex.what() << std::endl;
        return;
      } 
      refXml->get_widget("SaveFileChooser", saveFileChooser);
      refXml->get_widget("dialog-action_area5", saveFileChooserButtonBox);
      refXml->get_widget("OKButton", saveFileChooserOKButton);
      
      saveFileChooser->set_transient_for(*this);              
      // add the filters
      Gtk::FileFilter filter_pcal;
      filter_pcal.set_name(PCAL_FILTER_NAME);
      filter_pcal.add_pattern(PCAL_FILTER_EXT);
      saveFileChooser->add_filter(filter_pcal);
      Gtk::FileFilter filter_any;
      filter_any.set_name("Any files");
      filter_any.add_pattern("*");
      saveFileChooser->add_filter(filter_any);
    }            

    if(current_file!="") {
        saveFileChooser->set_filename(Glib::filename_from_utf8(current_file));
    }

    // rearrange the buttons according to the options
    if(Gtk::Dialog::alternative_button_order(get_screen())) {
        saveFileChooserButtonBox->reorder_child(*saveFileChooserOKButton, 0);
    } else {
        saveFileChooserButtonBox->reorder_child(*saveFileChooserOKButton, 1);
    }
      
    int result = saveFileChooser->run();
    saveFileChooser->hide();

    //Handle the response:
    switch(result)
    {
      case(Gtk::RESPONSE_OK):
      {
          Glib::ustring selected_file = Glib::filename_to_utf8(saveFileChooser->get_filename());
          if(selected_file != "") {
              // check if need to add the extention              
              Glib::ustring ext_str = PCAL_FILTER_EXT;
              ext_str = ext_str.substr(1);
              if(selected_file.length() < ext_str.length()) {
                  selected_file=selected_file+ext_str;
              } else 
              if(selected_file.substr(selected_file.length() - ext_str.length()) != ext_str) {
                  selected_file=selected_file+ext_str;
              }
                  
              // overwrite prompt
              std::ifstream in_file(saveFileChooser->get_filename().c_str());
              if(in_file) {
                  in_file.close();
    
                  Gtk::MessageDialog *m_dialog = new class Gtk::MessageDialog(
                              Glib::ustring(gettext("Overwrite existing file: ")) +
                              selected_file + "?",
                              false,
                              Gtk::MESSAGE_QUESTION,
                              Gtk::BUTTONS_YES_NO,
                              true);  
                  m_dialog->set_transient_for(*this);
      
                  int result = m_dialog->run();
                  delete m_dialog;
      
                  if(result != Gtk::RESPONSE_YES) return;
              }
    
              if(engine->saveToFile(selected_file)) {
                  current_file = selected_file;
                  updateWindowTitle();
              } else {              
                  Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
                              gettext("Can not save to file: \n")+
                              current_file,
                              false,
                              Gtk::MESSAGE_ERROR,
                              Gtk::BUTTONS_OK,
                              true);    
                  edialog->set_transient_for(*this);
                  edialog->run();
                  delete edialog;                 
              }
          }
          break;
      }
    }        
    updateWindowTitle();
}

void PCalMainWindow::updateWindowTitle()
{
    if(current_file == "")
    set_title(Glib::ustring(APP_NAME)+ " - " + gettext("new"));
    else
    set_title(Glib::ustring(APP_NAME)+ " - " + current_file);
}

bool PCalMainWindow::checkModified()
{
    if(engine->modified) {
        Gtk::MessageDialog *dialog = new class Gtk::MessageDialog(
                    gettext("Do you want to save changes?"),
		            false,
                    Gtk::MESSAGE_QUESTION,
                    Gtk::BUTTONS_YES_NO,
                    true);  
        dialog->set_transient_for(*this);
        dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
        if(Gtk::Dialog::alternative_button_order(get_screen())) {
            int order[3] = {Gtk::RESPONSE_YES, Gtk::RESPONSE_NO, Gtk::RESPONSE_CANCEL};
            dialog->set_alternative_button_order_from_array(order);
        }

        int result = dialog->run();
        delete dialog;

        switch(result)
        {
            case(Gtk::RESPONSE_YES):
            {
                on_save_item();
                if(engine->modified) return false;
                break;
            }
            case(Gtk::RESPONSE_NO):
            {
                break;
            }
            default:
            {
              return false;
            }
        }        
    }
    return true;
}

void PCalMainWindow::on_calendar_click(GdkEventButton* event)
{    
  keyboardSelected = false;
  if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) )
  {
      //Get the popup menu
      if(pPopupMenu == 0) {
          Glib::RefPtr<Gnome::Glade::Xml> refXml;
          try
          {
            refXml = Gnome::Glade::Xml::create(glade_interface_path, "calendarPopupMenu");
          }
          catch(const Gnome::Glade::XmlError& ex)
          {
            std::cerr << ex.what() << std::endl;
            return;
          }  
     
          refXml->get_widget("calendarPopupMenu", pPopupMenu);
          if(pPopupMenu)
          {
            //connect the popup items clicks
            Gtk::MenuItem* pMenuItem = 0;
            refXml->get_widget("addMenstruationPopupItem", pMenuItem);
            if(pMenuItem) {
                pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_add_menstruation_item));
                addPopupMenuItem = pMenuItem;
            }
            
            pMenuItem = 0;
            refXml->get_widget("deleteMenstruationPopupItem", pMenuItem);
            if(pMenuItem) {
                pMenuItem->signal_activate().connect( sigc::mem_fun(*this, &PCalMainWindow::on_delete_menstruation_item));
                deletePopupMenuItem = pMenuItem;
            }
            pPopupMenu->popup(event->button, event->time);
          }
      } else {
          pPopupMenu->popup(event->button, event->time);
      }
  }
  return;
}

void PCalMainWindow::on_add_menstruation_item()
{
    engine->addStartDate(*model_current_date);
    refreshCalendarView();
}
void PCalMainWindow::on_add_pregnancy_item()
{
    Glib::Date pdate;
    Glib::Date ldate;
    bool selected = false;
    // may we use the previous records to define the date?
    if(engine->getLastMenstruationDate(ldate)) {
       selected = dateSelector.getDateSelection(this, pdate,
            gettext("Pregnancy Starting Date Selection"),
            gettext("<b>Specify the first day of the last menstruation:</b>"),
            &ldate);
    } else {
        selected = dateSelector.getDateSelection(this, pdate,
            gettext("Pregnancy Starting Date Selection"),
            gettext("<b>Specify the first day of the last menstruation:</b>"));
    }   
    if(selected) {
        if(pdate.valid()) {
            engine->addPregnancyDate(pdate);
            refreshCalendarView();
        }
    }
}
void PCalMainWindow::on_add_birth_item()
{
    engine->addBirthDate(*model_current_date);
    refreshCalendarView();
}

void PCalMainWindow::on_delete_menstruation_item()
{
    engine->removeDateRecord(*model_current_date);
    refreshCalendarView();
}

void PCalMainWindow::refreshCalendarView()
{
    Glib::RefPtr<MonthCalendarModel> tModel = 
      Glib::RefPtr<MonthCalendarModel>::cast_dynamic(calendarTreeView->get_model());
    tModel->refresh();
    int start_day = (calOptions->firstDay == Glib::Date::SUNDAY) ? 0:1;
 
    for(int i=start_day;i<start_day+7;i++) {
        Gtk::TreeViewColumn* pColumn = calendarTreeView->get_column(i-start_day);
        if(pColumn)
        {
            pColumn->set_title(WDdays[i]);
        }
    }
    
    activateDate(* model_current_date);
    refreshStatistics();
}

Glib::ustring wrapBold(const Glib::ustring s) {
    Glib::ustring tmpstr = "<b>";
    tmpstr+=s;
    return tmpstr+="</b>";
}

void PCalMainWindow::refreshStatistics()
{
    
    Glib::ustring statusString = "";
    Glib::ustring tmpstr;
    char tmpchars[10];
 
    snprintf(tmpchars,10,"%u",engine->periodsCount);
    tmpstr.assign(tmpchars);
    statusString+=wrapBold(gettext("Periods: ")) + tmpstr + ", ";

    snprintf(tmpchars,10,"%u",engine->minLength);
    tmpstr.assign(tmpchars);
    statusString+=wrapBold(gettext("Min: ")) + tmpstr + ", ";
    
    snprintf(tmpchars,10,"%u",engine->avgLength);
    tmpstr.assign(tmpchars);
    statusString+=wrapBold(gettext("Avg: ")) + tmpstr + ", ";

    snprintf(tmpchars,10,"%u",engine->maxLength);
    tmpstr.assign(tmpchars);
    statusString+=wrapBold(gettext("Max: ")) + tmpstr + ", ";

    switch(engine->calMethodAccuracy) {
        case 0: {
            tmpstr = "<span foreground=\"#046700\">";
            tmpstr+=gettext("High");
            tmpstr+="</span>"; 
            break;
        }
        case 1: {
            tmpstr = "<span foreground=\"#83A110\">";
            tmpstr+=gettext("Medium");
            tmpstr+="</span>"; 
            break;
        }
        case 2: {
            tmpstr = "<span foreground=\"#C10A1D\">";
            tmpstr+=gettext("Low");
            tmpstr+="</span>"; 
            break;
        }
        default: {
            tmpstr = "<span foreground=\"#000000\">";
            tmpstr+=gettext("No Data");
            tmpstr+="</span>";
        }
    }
    statusString+=wrapBold(gettext("Accuracy: ")) + tmpstr;
    statusLabel->set_label(statusString);
}

void PCalMainWindow::activateDate(const Glib::Date selDate)
{
    PCalDayInfo info = engine->getDayInfo(selDate);
    dayInfoLabel->set_text(selDate.format_string("%x"));

    if(info.day_num == -1) {        
        periodDayNumberLabel->set_text("-");
        fertilityLabel->set_text("-");
        badFeelingsLabel->set_text("-");
        predictionLabel->set_label("");
        infoNotebook->set_current_page(0);
    } else {
        if(info.birth) {
            infoNotebook->set_current_page(2);
        } else
        if(info.pregnancy) {
            Glib::ustring tmpstr2;
            char tmpchars2[3];    
            snprintf(tmpchars2,3,"%u",(info.day_num/7+1));
            tmpstr2.assign(tmpchars2);
            pregnancyWeekLabel->set_text(tmpstr2);
            Glib::Date birthDay = selDate;
            birthDay.add_days(REGULAR_PREGNANCY_DAYS - info.day_num);
            estimatedBirthLabel->set_text(birthDay.format_string("%x"));
            infoNotebook->set_current_page(1);
        } else {
            infoNotebook->set_current_page(0);
    
            Glib::ustring tmpstr;
            char tmpchars[3];    
            snprintf(tmpchars,3,"%u",info.day_num+1);
            tmpstr.assign(tmpchars);
            periodDayNumberLabel->set_text(tmpstr);
            
            if(info.fertile) fertilityLabel->set_text(gettext("Yes"));
            else fertilityLabel->set_text(gettext("No"));
            if(info.bad_feel) badFeelingsLabel->set_text(gettext("Yes"));
            else badFeelingsLabel->set_text(gettext("No"));    
        }
        if(info.estimate) predictionLabel->set_label(
              Glib::ustring("<b>") + gettext("This is a prediction") + "</b>");
        else predictionLabel->set_label("");
    }
    PCalDayInfo day_info = engine->getDayInfo(*model_current_date);
 
    if(addPopupMenuItem && deletePopupMenuItem) {
        if(((day_info.day_num == 0) || 
             day_info.pregnancy ||
             day_info.birth)
          && !day_info.estimate) {
            addPopupMenuItem->hide();
            deletePopupMenuItem->show();
        } else {
            addPopupMenuItem->show();
            deletePopupMenuItem->hide();
        }
    }
    if(addMenstruationMenuItem &&  deleteMenstruationMenuItem) {
        if(((day_info.day_num == 0)  || 
             day_info.pregnancy ||
             day_info.birth)
          && !day_info.estimate) {
            addMenstruationMenuItem->set_sensitive(false);
            addPregnancyMenuItem->set_sensitive(false);
            addBirthMenuItem->set_sensitive(false);
            deleteMenstruationMenuItem->set_sensitive(true);
        } else {
            addMenstruationMenuItem->set_sensitive(true);
            addPregnancyMenuItem->set_sensitive(true);
            addBirthMenuItem->set_sensitive(true);
            deleteMenstruationMenuItem->set_sensitive(false);
        }        
    }
    calModel->updateSelection();
    
    // show the notes for the date if any
    // set the notes components state
    // are there notes for the current date?
    Glib::ustring note_text="";
    if(!engine->getDateNote(*model_current_date, note_text)) {
        // there are no notes for the current date
        // remove selection from the notes tree
        if(!activatingNote) {
            activatingNote = true;
            notesTreeView->get_selection()->unselect_all();
            activatingNote = false;
        }
        resettingNoteText = true;
        notesTextView->get_buffer()->set_text("");
        resettingNoteText = false;
        notesTextView->set_sensitive(false);
        noteAppendButton->set_sensitive(true);
        noteRemoveButton->set_sensitive(false);
        noteAppendMenuItem->set_sensitive(true);
        noteRemoveMenuItem->set_sensitive(false);
    } else {
        // there are notes for the current date
        // select the corresponding line in the list
        if(!activatingNote) {
            activatingNote = true;
            Gtk::TreeIter inote = notesModel->children().begin();
            while(inote!=notesModel->children().end()) {
                if((*inote)[notesColumns->m_col_date] == *model_current_date) break;
                inote++;        
            }

            if(!notesTreeView->get_selection()->is_selected(inote)) {           
                notesTreeView->get_selection()->select(inote);
            }
            activatingNote = false;
        }
        resettingNoteText = true;
        notesTextView->get_buffer()->set_text(note_text);
        resettingNoteText = false;
        notesTextView->set_sensitive(true);
        noteAppendButton->set_sensitive(false);
        noteRemoveButton->set_sensitive(true);
        noteAppendMenuItem->set_sensitive(false);
        noteRemoveMenuItem->set_sensitive(true);
    }
    activatingNote = false;
}

bool PCalMainWindow::on_delete_event(GdkEventAny* event)
{
    // do not allow to exit if there are unsaved changes
    // or the user have confirmed the changes lost
    return !checkModified();
}

void PCalMainWindow::on_quit_item()
{
    // do not allow to exit if there are unsaved changes
    // or the user have confirmed the changes lost
    if(checkModified()) {
        hide();
    }
}

void PCalMainWindow::nextMonth() {
    if(!(
        (cal_current_date.get_month() != model_current_date->get_month()) &&
        (cal_current_date < *model_current_date))) {
        *model_current_date = (*model_current_date).add_months(1);
    }
    refreshCalendarView();
}
void PCalMainWindow::nextYear() {
    *model_current_date = (*model_current_date).add_years(1);
    refreshCalendarView();
}
void PCalMainWindow::prevMonth() {
    if(!(
        (cal_current_date.get_month() != model_current_date->get_month()) &&
        (cal_current_date > *model_current_date))) {
	// Glib doesn't like year 0
        if(!((model_current_date->get_month() == 1) && (model_current_date->get_year() == 1))) {
            *model_current_date = (*model_current_date).subtract_months(1);
        }

    }
    refreshCalendarView();
}
void PCalMainWindow::prevYear() {
    if((model_current_date->get_year() > 1))
        *model_current_date = (*model_current_date).subtract_years(1);

    refreshCalendarView();
}
void PCalMainWindow::today() {
    *model_current_date = *model_today_date;
    refreshCalendarView();
}

/* void PCalMainWindow::on_calendar_key_press(GdkEventKey* event)
{
    Glib::Date oldDate = *model_current_date;
    switch(event->keyval)
    {
        case GDK_Up: *model_current_date = (*model_current_date).subtract_days(7); break;
        case GDK_Down: *model_current_date = (*model_current_date).add_days(7); break;
        case GDK_Left: *model_current_date = (*model_current_date).subtract_days(1); break;
        case GDK_Right: *model_current_date = (*model_current_date).add_days(1); break;
        case GDK_Page_Up: prevMonth(); return;
        case GDK_Page_Down: nextMonth(); return;
        default: return;
    }
    if(oldDate != *model_current_date) {
        keyboardSelected = true;
    
        // no really need to refresh everything here - just too lazy to implemt
        // smart refreshes (is this really needed here?)
        if(cal_current_date.get_month()!=model_current_date->get_month()) {
            refreshCalendarView();
        } else {
            activateDate(*model_current_date);
        }
    }
}
*/
void PCalMainWindow::start_with_file(Glib::ustring fname)
{
   if(engine->loadFromFile(fname)) {
        current_file = fname;
        updateWindowTitle();
        reload_notes_tree();
        refreshCalendarView();
    } else {
        Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
                    gettext("Can not load file: \n")+
                    fname,
		            false,
                    Gtk::MESSAGE_ERROR,
                    Gtk::BUTTONS_OK,
                    true);    
        edialog->set_transient_for(*this);
        edialog->run();
        delete edialog;                 
    }
}

void PCalMainWindow::show_help(const Glib::ustring keyword)
{
    //!!! place the real help showing code here
    Gtk::MessageDialog *edialog = new class Gtk::MessageDialog(
               "Help topic: " + keyword,
               false,
               Gtk::MESSAGE_INFO,
               Gtk::BUTTONS_OK,
               true);
   edialog->set_transient_for(*this);
   edialog->run();
   delete edialog;   
}

void PCalMainWindow::show_help_contents() 
{
    show_help();
}

bool PCalMainWindow::select_note_function(const Glib::RefPtr<Gtk::TreeModel>& model,
const Gtk::TreeModel::Path& path, bool)
{
    if(activatingNote) return true;
    activatingNote = true;
    Gtk::TreeRow row = *(model->get_iter(path));
    *model_current_date = row[notesColumns->m_col_date];
    refreshCalendarView();
    activatingNote = false;
    return true;
}

void PCalMainWindow::on_note_append_item()
{
    // just make this safer - check if the note already exists
    if(!engine->existsDateNote(*model_current_date)) {
        Gtk::TreeModel::Row row = *notesModel->append();
        row[notesColumns->m_col_label] = model_current_date->format_string("%x");
        row[notesColumns->m_col_date] = *model_current_date;
        engine->setDateNote(*model_current_date, "");
        notesTreeView->get_selection()->select(row);
        notesTextView->grab_focus();
    }
}

void PCalMainWindow::on_note_remove_item()
{
    Gtk::MessageDialog *dialog = new class Gtk::MessageDialog(
                gettext("Are you sure to delete the note for ")+
                    model_current_date->format_string("%x"),
                false,
                Gtk::MESSAGE_QUESTION,
                Gtk::BUTTONS_YES_NO,
                true);  
    dialog->set_transient_for(*this);

    int result = dialog->run();
    delete dialog;

    if(result == Gtk::RESPONSE_YES) {
        engine->deleteDateNote(*model_current_date);
        // Reloading everything is really stupid, but have no
        // time to deal with this more
        // A great point for the optimization
        reload_notes_tree();
        resettingNoteText = true;
        notesTextView->get_buffer()->set_text("");
        resettingNoteText = false;
        notesTextView->set_sensitive(false);
        noteAppendButton->set_sensitive(true);
        noteRemoveButton->set_sensitive(false);
        noteAppendMenuItem->set_sensitive(true);
        noteRemoveMenuItem->set_sensitive(false);
    }
}

void PCalMainWindow::reload_notes_tree()
{
    // load the notes list into the tree control
    const std::map<Glib::Date, Glib::ustring>& notes_ref = engine->getDateNotes();
    
    notesModel->clear();
    
    std::map<Glib::Date, Glib::ustring>::const_iterator p;
    for(p=notes_ref.begin();p!=notes_ref.end();p++) {
        Gtk::TreeModel::Row row = *notesModel->append();
        row[notesColumns->m_col_label] = p->first.format_string("%x");
        row[notesColumns->m_col_date] = p->first;
    }
}

void PCalMainWindow::refreshToolbar() 
{
    Gtk::ToolbarStyle tStyle;    
    if(calOptions->toolbarStyle == "Icons") {
        tStyle = Gtk::TOOLBAR_ICONS;
    } else 
    if(calOptions->toolbarStyle == "Text") {
        tStyle = Gtk::TOOLBAR_TEXT;
    } else 
    if(calOptions->toolbarStyle == "Both") {
        tStyle = Gtk::TOOLBAR_BOTH;
    } else 
    if(calOptions->toolbarStyle == "Both Horizontal") {
        tStyle = Gtk::TOOLBAR_BOTH_HORIZ;
    } else {
        // System Default
        tStyle = get_settings()->property_gtk_toolbar_style();
    }    
    for(int i=0; i<2; i++) {    
        mainToolbars[i]->set_toolbar_style(tStyle);
    }
}

void PCalMainWindow::on_note_text_changed()
{
    // save the changed text to the data model
    if(!resettingNoteText) {
        engine->setDateNote(*model_current_date, notesTextView->get_buffer()->get_text());    
    }
}

void PCalMainWindow::on_jump_to_date_item()
{
    Glib::Date tmpDate;

    if(dateSelector.getDateSelection(this, tmpDate,
            gettext("Select Date"),
            gettext("<b>Specify the date to move to:</b>"),
            model_current_date)) {
        *model_current_date = tmpDate;
        refreshCalendarView();
    }
}

int PCalMainWindow::treeModelDatesCompare(const Gtk::TreeModel::iterator& item1, 
                                          const Gtk::TreeModel::iterator& item2)
{    
    Glib::Date date1 = (*item1)[notesColumns->m_col_date];
    Glib::Date date2 = (*item2)[notesColumns->m_col_date];
    if(!date1.valid()) {
        return -1;
    } else
    if(!date2.valid()) {
        return 1;
    } else {
        return date1.compare(date2);
    }
}
